﻿/*
 * This file is part of AV Orchid.
 * Copyright (c) 2014, Dmitri R. Kuvshinov <evetro.here@gmail.com>
 * 
 * Permission to use, copy, modify, and/or distribute this software for
 * any purpose with or without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all copies.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
/**
 * @file APlot2D.hpp
 * @brief фасад для удобного добавления графиков (неявно используется глобальный объект сцены)
 */
#pragma once
#include "PlotVertices2.hpp"
#include <avo/Color.hpp>
#include <string>
#include <forward_list>
using avo::Color;

inline Color rgb(float red, float green, float blue)
{
  return Color{red, green, blue};
}

/// Класс, объекты которого добавляют график (название, цвет, толщину линий).
/// Название с маленькой буквы, чтобы имитировать вызов функции (конструктор).
class plot
{
public:
  plot(std::string title, Color color, float line_width = 2.f)
    : title(std::move(title)),
      color(color),
      line_width(line_width)
  {}

  ~plot(); // собственно добавляет график, определён в APlot2D.cpp

  // branch является отражением plot_vertices2
  
  /**
   * @param fxy : R -> R2 непрерывная на [t0, t1] функция, возвращающая точку плоскости для некоторого значения параметра
   * @param t0 начальное значение параметра
   * @param t1 конечное значение параметра
   * @param min_t_step минимальный возможный шаг параметра
   * @param max_t_step максимальный возможный шаг параметра
   * @param tol (условная) допустимая относительная погрешность на звено (оценочная длина куска графика / длина отрезка)
   */
  template <class FXY>
  plot& branch(FXY fxy, Scalar t0, Scalar t1, Scalar min_t_step, Scalar max_t_step, Scalar tol)
  {
    branches.emplace_front(plot_vertices2(fxy, t0, t1, min_t_step, max_t_step, tol));
    return *this;
  }

  /**
   * @param fx : R -> R непрерывная на [t0, t1] функция, возвращающая x-координату для некоторого значения параметра
   * @param fy : R -> R непрерывная на [t0, t1] функция, возвращающая y-координату для некоторого значения параметра
   * @param t0 начальное значение параметра
   * @param t1 конечное значение параметра
   * @param min_t_step минимальный возможный шаг параметра
   * @param max_t_step максимальный возможный шаг параметра
   * @param tol (условная) допустимая относительная погрешность на звено (оценочная длина куска графика / длина отрезка)
   */
  template <class FX, class FY>
  plot& branch(FX fx, FY fy, Scalar t0, Scalar t1, Scalar min_t_step, Scalar max_t_step, Scalar tol)
  {
    branches.emplace_front(plot_vertices2(fx, fy, t0, t1, min_t_step, max_t_step, tol));
    return *this;
  }

private:
  std::forward_list<Points_2> branches;
  std::string title;
  Color color;
  float line_width;
};

/// запуск программы APlot2D с параметрами командной строки
int run_aplot2d(int argc, char *argv[]);

/// запуск программы APlot2D без параметров командной строки
inline int run_aplot2d()
{
  char nothing[] = { '\0' };
  char *argv[] = { nothing, nothing };
  return run_aplot2d(1, argv);
}
