﻿// Проект APlot2D

// Параметры plot:
// название графика, цвет графика, толщина линии графика в пикселях.
//
// Для каждой ветви (заданной своим набором функций и диапазоном параметра) следует вызвать branch
// с параметрами:
// одна (возвращающая точку) или две (возвращающие числа -- значения координат x и y) функции одного параметра,
// диапазон изменения параметра [t0, t1], задающего ветвь,
// диапазон, ограничивающий шаг по параметру при вычислении вершин графика [min_t_step, max_t_step],
// максимум на оценку относительной погрешности по длине отрезка аппроксимирующей ломаной (число > 0).
#include "APlot2D.hpp"
using namespace std;


int main()
{
  // пример 1, одна функция p(t) возвращает точку на плоскости
  plot("Helix", avo::VIOLET, 2.f)
    .branch
  (
    [](Scalar t) -> Point_2
    {
      const auto coef = (10. - 10./sqrt(t + 1.));
      return coef * Point_2(sin(t), cos(t));
    },
    0., 150., 0.05, 0.5, 0.001
  );

  // пример 2, две функции: x(t) и y(t)
  plot("t sin 1/t", avo::YELLOW, 2.f)
    .branch
  (
    [](Scalar t) { return t; },
    [](Scalar t) { return t * sin(1./t); },
    -8., 8., 0.0001, 0.1, 0.00001
  );

  // пример 3, две ветви, цвет компонентами RGB (красный, зелёный, синий в диапазоне [0, 1])
  plot("quasihyperbola", rgb(0.8f, 0.4f, 0.1f))
    .branch
  (
    [](Scalar t) { return t; },
    [](Scalar t) { return 1./sqrt(t); },
    0.01, 10., 1e-5, 1., 1e-5
  )
    .branch
  (
    [](Scalar t) { return t; },
    [](Scalar t) { return -pow(-t, -1./3.); },
    -10., -0.001, 1e-5, 1., 1e-5
  );

  // запустить визуализацию
  return run_aplot2d();
}
