﻿/*
 * This file is part of AV Orchid.
 * Copyright (c) 2016, Dmitri R. Kuvshinov <evetro.here@gmail.com>
 * 
 * Permission to use, copy, modify, and/or distribute this software for
 * any purpose with or without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all copies.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
/**
 * Пример: CPlot2D
 * Автор: Кувшинов Д.Р. https://evetro.wordpress.com/
 * Используемые библиотеки: FreeGlut, OpenGL.
 *
 * Асинхронное построение графиков.
 *
 * Управление "камерой": стрелки и WASD, +/- Z/X для увеличения/уменьшения масштаба.
 * Колёсико мыши изменяет масштаб (как +/-).
 * HOME -- вернуть камеру в положение по умолчанию.
 * ESC -- выход из программы.
 */
#include <avo/Timer.hpp>
#include "Plotting2D.hpp"
#include <cmath>
#include <iostream>


int main()
{
  setup_plotting_window("CPlot2D", 700, 700);
  plotting_view(-10., -10., 50., 50.);

  while (true)
  {
    start_plotting();
    for (int i = 10; i < 100000; ++i)
    {
      const double phi = i * 0.1;
      const double r = 100. / sqrt(sqrt(double(i)));
      plot(r * cos(phi) + 10., r * sin(phi));
      
      // Форсировать обновление области (и синхронизацию) через каждые 100 итераций.
      if (i % 100 == 0)
        plotting_view_all();
      //avo::sleep(0.001);
    }

    avo::sleep(1.);

    if (!start_plotting(avo::AZURE, 5.f, avo::PINK, 3.f))
      break;

    for (int i = 10; i < 1000; ++i)
    {
      const double phi = -i * 0.1;
      const double r = 100. / sqrt(sqrt(double(i)));
      plot(r * cos(phi) - 10., r * sin(phi));
      plotting_view_all();
      avo::sleep(0.001);
    }

    std::cout << "done" << std::endl;
    // 10 секунд, пока нет вызовов plotting_view_all, можно перемещать область
    for (int i = 0; i < 10; ++i)
    {
      if (is_already_closed())
        break; // выйти, если пользователь закрыл окно
      avo::sleep(1.); // ждать 1 секунду
    }

    if (!clear_plotting())
      break;
  }
  return 0;
}
