﻿/*
 * This file is part of AV Orchid.
 * Copyright (c) 2013, Dmitri R. Kuvshinov <evetro.here@gmail.com>
 * 
 * Permission to use, copy, modify, and/or distribute this software for
 * any purpose with or without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all copies.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
/**
 * Пример: Pencil
 * Автор: Кувшинов Д.Р. https://evetro.wordpress.com/
 *
 * Данный файл предоставляет интерфейс IBounded_2D_cellular_automaton и две реализации игры "Жизнь" Конвея. 
 */
#pragma once
#include <cstddef>
#include <utility> // std::pair
#include <avo/Pixel.hpp>


/// интерфейс "клеточного автомата", ("interface" --> ведущая буква I в названии),
// в C++ может быть представлен абстрактным классом, не содержащим никаких деталей реализации
struct IBounded_2D_cellular_automaton
{
  /// тип размера
  using Index = std::ptrdiff_t;
  /// пара размеров (ширина и высота)
  using Extents = std::pair<Index, Index>;
  /// деструктор
  virtual ~IBounded_2D_cellular_automaton() {}
  /// название автомата
  virtual const char* name() = 0;
  /// обновить размеры поля
  virtual void reshape(Index width, Index height) = 0;
  /// получить текущие размеры поля
  virtual Extents extents() const = 0;
  /// построить исходное поле на основе 32-битной картинки (чёрный => 0, нечёрный => 1)
  virtual void from_pixels(const avo::Pixel *src) = 0;
  /// следующий шаг по времени, сформировать 32-битную картинку в dst
  virtual void next(avo::Pixel *dst) = 0;
};


// следующие функции возвращают объекты, предоставляющие интерфейс ICellularAutomaton,
// реализующие игру "Жизнь" Конвея

/// вычисляет поэлементные суммы ("наивный алгоритм"), использует OpenMP для распараллеливания
IBounded_2D_cellular_automaton* new_Conways_Life_cellwise_summator();

/// вычисляет построчные суммы, использует OpenMP для распараллеливания
IBounded_2D_cellular_automaton* new_Conways_Life_line_summator();

/// работает аналогично new_Conways_Life_line_summator, но результирующий объект использует SSE3
IBounded_2D_cellular_automaton* new_Conways_Life_line_summator_SSE3();
