AV Orchid
=========

v.2.9.0


Набор учебных примеров, использующих [FreeGlut](http://freeglut.sourceforge.net/), [GLEW](http://glew.sourceforge.net/), [OpenGL](http://www.khronos.org/opengl) ([wiki](http://www.opengl.org/wiki/Main_Page)), [OpenMP](http://openmp.org/wp/).
Примеры не "заточены" на графические технологии (как правило, используется минимальный OpenGL 1.1).

Примеры не зависят друг от друга. Принцип [DRY](http://ru.wikipedia.org/wiki/Don%27t_repeat_yourself) нарушается, чтобы упростить общую структуру.

Примеры значительно различаются по сложности.

Разрабатывается в MSVC2013.
Теоретически код является кроссплатформенным (требует уровень поддержки C++11 не ниже MSVC2013), однако это не проверялось.

[Альтернативы FreeGlut](http://www.opengl.org/resources/libraries/windowtoolkits/) (все перечисленные библиотеки являются кроссплатформенным открытым ПО):
	
- практически прямая альтернатива [GLFW](http://www.glfw.org/) не обременена багажом совместимости с GLUT; недостаток: позволяет создать только одно окно и один контекст OpenGL.
- более развитая (но всё же относительно небольшая) объектно-ориентированная C++ библиотека [SFML](http://www.sfml-dev.org/).
- более низкоуровневая и развитая библиотека [SDL](http://www.libsdl.org/), также предоставляет средства вывода звука.

Для работы с 2-3-4-мерными векторами, матрицами и кватернионами хорошие возможности предоставляет C++ библиотека [GLM](http://glm.g-truc.net/), которая имитирует [GLSL](http://en.wikipedia.org/wiki/OpenGL_Shading_Language).

Для автоматической генерации документации по исходному коду рекомендуется использовать [Doxygen](http://www.stack.nl/~dimitri/doxygen/). Пример -- Zmeika.

На данный момент проект по большей части заморожен.


## Состав

### Include/Source/Lib

Вспомогательные файлы (файлы сторонних библиотек включены в сборку).

### BasicFrame

Каркас FreeGlut-приложения.

### Pixels

Демонстрация вывода произвольной пиксельной графики средствами OpenGL.

### GLPrimitives

Демонстрация различных геометрических примитивов OpenGL 1.x, а также моделей затенения flat и smooth.

### GLPrimitivesVA

Аналог GLPrimitives, использующий массивы вершин вместо glBegin/glEnd.

### GLPrimitivesVB

Аналог GLPrimitivesVA, использующий буфер в памяти GPU для размещения массивов вершин (в духе OpenGL 1.5).

### Plasma

Эффект "[плазмы](http://en.wikipedia.org/wiki/Plasma_effect)".
Демонстрирует использование табличных функций (без интерполяции).

### Collision2D

Симуляция движения и столкновения двух тяжёлых кружков в плоскости.
Кружки можно перетаскивать мышью.

### Collision3D

Симуляция движения и столкновения тяжёлых шаров в пространстве.
Шары притягиваются друг к другу под действием классического закона тяготения.

### ColorClock

Часы.

### Snow2D

Рисует падающий цветной "снег".

### GLSLPlasma

Эффект плазмы, реализованный в виде пиксельного шейдера [GLSL](http://www.opengl.org/documentation/glsl/), требует поддержки OpenGL 2.0 для запуска ([GLSL 1.10](http://www.opengl.org/registry/doc/GLSLangSpec.Full.1.10.59.pdf)).
Файлы с пиксельными шейдерами имеют названия вида d.glsl.frag, где d -- цифра. Переключение в программе выполняется нажатием соответствующей цифры.

### Manipulator

Имитация движения манипулятора.

### PencilPaleo

Упрощённый вариант Pencil (см. ниже).

### Pencil

В примере пользователь может рисовать белым "карандашом" на чёрном "холсте"
с помощью мыши (зажимая левую кнопку). Реализация [игры "Жизнь"](http://ru.wikipedia.org/wiki/%D0%96%D0%B8%D0%B7%D0%BD%D1%8C_%28%D0%B8%D0%B3%D1%80%D0%B0%29) Джона Конвея.

### APlot2D

Рисование графиков на плоскости, заданных парами функций (x(t), y(t)) для t из заданных отрезков.

### BPlot2D

Визуализация индикаторных функций множеств на плоскости. Базируется на APlot2D и GLSLPlasma. Требует поддержки OpenGL 2.1 (GLSL 1.20).

### BPlot2Dt

Клон BPlot2D с переменной время (t) в шейдере.

### CPlot2D

Визуализация графиков в параллельном потоке (одновременно с процессом порождения узлов графика).

### FIRFilter

Визуализация работы КИХ-фильтра (в данном случае -- операции численного дифференцирования сигнала по пяти последним узлам).

### GLUPoly

Демонстрация использования [GLU](http://www.opengl.org/registry/doc/glu1.3.pdf) для [триангуляции многоугольников](http://en.wikipedia.org/wiki/Polygon_triangulation).

### SortVis

Визуализатор различных алгоритмов сортировки.

### AStarVis

Визуализатор [алгоритма поиска на графе A*](http://ru.wikipedia.org/wiki/%D0%90%D0%BB%D0%B3%D0%BE%D1%80%D0%B8%D1%82%D0%BC_%D0%BF%D0%BE%D0%B8%D1%81%D0%BA%D0%B0_A*).
Генерирует ландшафт на основе псевдослучайной карты высот.

### Zmeika

Игра "Змейка".

Более подробные описания размещены в [вики проекта](https://bitbucket.org/evetro/av-orchid/wiki/Home).
