﻿/*
 * This file is part of AV Orchid.
 * Copyright (c) 2013, Dmitri R. Kuvshinov <evetro.here@gmail.com>
 * 
 * Permission to use, copy, modify, and/or distribute this software for
 * any purpose with or without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all copies.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
/**
 * @file Game.cpp
 *
 * Реализация элементов Game.hpp.
 */
#include <algorithm>
#include <cstdlib>

#include "Game.hpp"
#include "ConfigFile.hpp"

using namespace std;

//
// Level

istream& operator>>(istream &is, Level &level)
{
  level = Level();
  return is >> level.start_length >> level.max_length >> level.speed
            >> level.max_movement >> level.penalty;
}


//
// Zmeika_game

Zmeika_game::Zmeika_game()
  : level(1),
    score(0),
    score_max(0),
    field(14, 14),
    time_step(0.2),
    snake(field)
{}

// прочитать описания уровней
void Zmeika_game::read_levels()
{
  Dictionary d = load_dictionary("levels.cfg");

  levels.clear ();
  const auto d_end = d.end();
  for (Index i = 1, iend = d.size(); i <= iend; ++i)
  {
    auto p = d.find(toString(i));
    if (p != d_end)
      levels.emplace_back(try_parse<Level>(p->second));
  }

  srand(try_get(d, "seed", 1337u));
  // очки за прохождение клеток с заданным номером
  move_score[0] = try_get(d, "movement_score",  1u);
  move_score[1] = 0; // collision is not paid for!
  move_score[2] = try_get(d, "rabbit_1_score", 20u);
  move_score[3] = try_get(d, "rabbit_2_score", 30u);
  move_score[4] = try_get(d, "rabbit_3_score", 50u);
  level_score   = try_get(d, "level_score",   100u);

  field = Field(try_get(d, "field_xsize", field.xsize()),
                try_get(d, "field_ysize", field.ysize()));
}


// поставить нового кролика
void Zmeika_game::new_rabbit(Cell rabbit_type)
{
  while (true) // зависнет, если нет пустых клеток
  {
    // способ взят с потолка, можно просто поставить rand() вместо c и d
    int a = std::rand(), b = std::rand(),
        c = (a * 101) ^ (b * 129),
        d = (a * 779) ^ (b * 133);

    // поставить кролика "случайно"
    Field::Position rabbit(1 + c % (field.xsize() - 2), 1 + d % (field.ysize() - 2));
    if (field[rabbit] == CELL_EMPTY)
    {
      field[rabbit] = rabbit_type;
      return;
    }
  }
}


// переключить уровень
void Zmeika_game::select_level(Index next_level_index)
{
  level = min(next_level_index, static_cast<Index>(levels.size()));
  const auto &next_level = levels[level - 1];
  snake.reset(next_level.start_length);
  time_step = 1.0 / next_level.speed;
  up(); // начальное направление движения змеи
  steps = 0;
}


// реакция на движение
bool Zmeika_game::moved_into(const Cell movement)
{
  const auto &cur_level = levels[level - 1];
  // очки за движение по пустым клеткам начисляются только до достижения некоторого максимального значения
  if (movement != CELL_EMPTY || steps++ < cur_level.max_movement)
    add_score(move_score[movement] * level);

  if (movement == CELL_SNAKE) // врезались
  {
    if (snake.length() != cur_level.start_length)
      sub_score(cur_level.penalty * level);
    return true;
  }

  if (movement >= CELL_RABBIT) // съели кролика... бедный кролик
  {
    // next level?
    if (snake.length() >= cur_level.max_length)
    {
      select_level(level + 1);
      add_score(level * level_score);
    }

    // да здравствует вечный кролик!
    new_rabbit(CELL_RABBIT + rand() % (CELL_TYPES - CELL_RABBIT));
  }

  return false;
}


// добавить очки
void Zmeika_game::add_score(Score amount)
{
  score += amount;
  if (score > score_max)
    score_max = score;
}

// снять очки
void Zmeika_game::sub_score(Score amount)
{
  score = score < amount? 0: score - amount;
}
