﻿/*
 * This file is part of AV Orchid.
 * Copyright (c) 2013, Dmitri R. Kuvshinov <evetro.here@gmail.com>
 * 
 * Permission to use, copy, modify, and/or distribute this software for
 * any purpose with or without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all copies.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
/**
 * @file Zmeika.cpp
 *
 * Файл содержит функцию main.
 */
/**
 * @mainpage Пример: Zmeika
 *
 * Автор: Кувшинов Д.Р. https://evetro.wordpress.com/
 *
 * Используемые библиотеки: FreeGlut, OpenGL.
 *
 * @image html Zmeika.png
 *
 *
 * Классическая игра, в которой на поле, разбитом на клетки, требуется собирать "кроликов", 
 * управляя движением "змеи" из клеток, и избегать столкновений со стенками и змеи с самой собой.
 *
 * Файлы:
 *   * levels -- описание уровней;
 *   * hall_of_fame -- "Hall of Fame".
 *
 * Экраны:
 *   1. Список лучших игроков.
 *   2. Игра.
 *   3. Ввод имени игрока.
 *
 * Нажатие:
 *   * W, A, S, D и стрелки -- движение (вверх, влево, вниз и вправо);
 *   * R -- сброс игры без сохранения очков;
 *   * SPACE/ENTER -- 1) начать игру, 2) продолжить/пауза;
 *   * ESC -- 1) выйти из программы, 2) прекратить игру.
 *
 */
// TASK: добавить препятствия
#include <GL/freeglut.h>
#include <string>
#include <iostream>

#include "GameScreen.hpp"
#include "EnterNameScreen.hpp"
#include "HallOfFameScreen.hpp"

#include "ConfigFile.hpp"


int main(int argc, char **argv)
{
  // базовая инициализация
  glutInit(&argc, argv); // обработка параметров командной строки

  // Создать окно с привязанным контекстом OpenGL.
  // Режим: 32-битный цвет RGBA, двойная буферизация DOUBLE.
  glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE);
  glutInitWindowPosition(0, 0); // координаты верхнего левого угла окна на экране
  glutInitWindowSize(714, 777); // полные ширина и высота окна
  glutCreateWindow("Zmeika");   // создать окно, в функцию передаётся заголовок создаваемого окна

  // старт
  std::cout << "Starting Zmeika\n";
  Game_screen gameScreen;
  Enter_name_screen enScreen;
  Hall_of_fame_screen hofScreen;
  Score last_sc = 0;

  // обработчик события "начало игры" переключает текущий экран на игровой
  hofScreen.set_game_started_handler
  (
    [&]() { gameScreen.select (); }
  );

  // обработчик события "игра окончена" запрашивает имя пользователя и переключает экран на hofScreen
  gameScreen.set_game_over_handler
  (
    [&](Score sc)
    {
      if (sc == 0)
        return hofScreen.select();

      enScreen.set_score(std::to_string(sc));
      last_sc = sc;
      enScreen.select();
    }
  );

  enScreen.set_name_entered_handler
  (
    [&](const std::string &name)
    {
      hofScreen.add_player(last_sc, name);
      hofScreen.select();
    }
  );

  // в начале игры показывает список лучших игроков
  hofScreen.select();
  glutMainLoop(); // вход в цикл обработки событий
  std::cout << "\nbye!";
  return 0;
}
